% all_candbbs is used only to get the image indices and nb of bboxes.

function S = classify_all_boxes(classifier, descriptors, all_candbbs)   
  pqs = descriptors.pqs;
  
  nchan = length(descriptors.channels);
    
  l2normsq_c = [];
  for i = 1:nchan
    l2normsq_c = [l2normsq_c, pqs(descriptors.z2c(i)).l2normsq];
  end  
  
  S = struct;
  if descriptors.cache_level == 0
    % load all codes and nb of pts
    data = get_sample_descriptors(descriptors, all_candbbs, 2);
  else  

    % compute all scores in mex
    imind = int32([all_candbbs.orgimi] - 1); 
    [all_scores, npts_stats] = descriptor_cache_mex('detect', ...
                                                    imind, classifier.alls_pq, ...
                                                    l2normsq_c);
  end
  
  di = 0; % how many columns of the descriptor matrix are processed so far
  for i = 1:length(all_candbbs)
    candbbs = all_candbbs(i);
    nbox = size(candbbs.bbuint8, 1);
    if descriptors.cache_level == 0  
      [scores, sumofnpts, maxofnpts] = compute_image_scores(classifier.alls_pq, descriptors, l2normsq_c, ...
                                                  data.codes(:, di + 1 : di + nbox), data.npts(:, di + 1 : di + nbox)); 
    else 
      scores = all_scores(di + 1 : di + nbox)'; 
      sumofnpts = npts_stats(di + 1 : di + nbox, 1); 
      maxofnpts = npts_stats(di + 1 : di + nbox, 1); 
    end

    
    % cancel scores for windows with too few descriptors
    scores(maxofnpts < 20) = -Inf;
    scores  = bsxfun(@plus, scores, classifier.allw_e.' * ones(1, nbox)).';
    
    S(i).orgimi = candbbs.orgimi;
    S(i).scores = scores;
    S(i).bbuint8 = candbbs.bbuint8;
    
    di = di + nbox;
  end
     
  
end


function [scores, sumofnpts, maxofnpts] = compute_image_scores(alls_pq, descriptors, ...
                                                  l2normsq_c, codes, npts)
    
  pqs = descriptors.pqs;
  nsq = pqs(1).nsq;
  nbox = size(codes, 2);
  nchan = length(descriptors.channels); 
  scores = zeros(1, nbox);
  sumofnpts = zeros(1, nbox);
  maxofnpts = zeros(1, nbox);
  q0 = 0;
    
  for chani = 1:nchan      
    for j = 1:nbox
      
      if 0
        % explicit loop (slow)
        
        tmp4bb = 0; 
        l2sq_cell = 0;
        pq = pqs(descriptors.z2c(chani));
        for q = 1:nsq  
          code = int32(codes(q0 + q, j)) + 1;
          tmp4bb = tmp4bb + alls_pq(code, q0 + q);
          l2sq_cell = l2sq_cell + pq.l2normsq(code, q);
        end
        
      else 
        
        % vector notation (a bit faster)
        idx = double(codes(q0 + (1:nsq), j)) + 256 * (q0 : q0 + nsq - 1)' + 1;
        tmp4bb = sum(alls_pq(idx));
        l2sq_cell = sum(l2normsq_c(idx));
        
      end
      
      np = npts(chani, j);
      sumofnpts(j) = sumofnpts(j) + np;
      maxofnpts(j) = max(maxofnpts(j), np);
      
      EPS = 1e-16;
      scores(j) = scores(j) + tmp4bb / max(EPS, sqrt(l2sq_cell));
    end

    q0 = q0 + nsq;
              
  end
  
end
