# stop at errors
set -e

# show commands
set -x 


# This works for a system-installed OpenCV. Add -I and -L options if
# it is installed elsewhere

opencvflags="-lopencv_core -lopencv_imgproc"


# hard to put openmp flags in a variable, so make a function 
function mex_openmp () {
    mex CFLAGS='$CFLAGS -fopenmp'  CXXFLAGS='$CXXFLAGS -fopenmp'    LDFLAGS='$LDFLAGS -fopenmp' "$@"
}


(
cd selectivesearch_uva
mex anigaussm/anigauss_mex.c anigaussm/anigauss.c -output anigauss
mex FelzenSegment/mexFelzenSegmentIndex.cpp -output mexFelzenSegmentIndex
)


(
cd myutils/imagedata/private
mex -I../../cpplib cn_imresize_opencv.cpp $opencvflags
)


(
cd myutils/miscutils/private/
mex_openmp -largeArrayDims -I../../cpplib  cn_sorttopk_mex.cpp ../../cpplib/cn_threads.cpp
)



(
cd myutils/imagedata/
mex_openmp  -I../cpplib -largeArrayDims cn_fvgmm_poolobjdesc_mex.cpp ../cpplib/cn_threads.cpp ../cpplib/cn_fvgmm.cpp
)

(
cd myutils/imagedata
mex -I../cpplib -largeArrayDims cn_fvgmm_reshape.cpp $opencvflags
)


(
cd myutils/clustering/private/
mex_openmp -I../../cpplib -largeArrayDims cn_kmeansparallel_mex.cpp ../../cpplib/cn_siginthandler.cpp ../../cpplib/cn_rand.cpp  ../../cpplib/cn_threads.cpp
)


(
cd myutils/clustering/productquantization/
mex_openmp -I../../cpplib -largeArrayDims pq_assign.cpp ../../cpplib/cn_threads.cpp
mex_openmp -I../../cpplib -largeArrayDims pq_l2normsq.cpp 
)



(
cd myutils/detectionutils/
mex_openmp -I../cpplib -largeArrayDims cn_nmsgreedy_mex.cpp -I../distancefunctions/private/
)

(
cd myutils/distancefunctions/private
mex_openmp -largeArrayDims -I../../cpplib cn_bboverlap_main_mex.cpp 
)


(
cd myutils/classifiers/private
mex_openmp -largeArrayDims -I../../cpplib  cn_pqliblinear_prepmex.cpp ../../cpplib/cn_pq.cpp  ../../cpplib/cn_threads.cpp ../../cpplib/cn_blas.cpp -lmwblas
mex -largeArrayDims cn_pqliblinear_mex.cpp  -I../../cpplib
) 


(
cd compression

gcc -fPIC -O -c blosc.c 
gcc -fPIC  -O -c blosclz.c 
gcc -fPIC  -O -c shuffle.c
gcc -fPIC  -Wall -fopenmp -O -c flexdata.c
gcc -fPIC  -O -c flexdata_client_server.c

mex compress_blosc_mex.c blosc.o blosclz.o shuffle.o
mex decompress_blosc_mex.c blosc.o blosclz.o shuffle.o

mex_openmp  descriptor_cache_mex.c flexdata.o flexdata_client_server.o blosc.o blosclz.o  shuffle.o 

gcc -fopenmp -o cache_server flexdata_client_server.c -DSERVER_EXECUTABLE flexdata.o blosc.o blosclz.o shuffle.o -lm
)

