#include <string.h>
#include <mex.h>
#include "blosc.h"


#define INPUT prhs[0]
#define OUTPUT plhs[0]


void mexFunction (int nlhs, mxArray *plhs[],
                  int nrhs, const mxArray*prhs[])

{
  if (!(nrhs == 1 && nlhs == 1)) 
    mexErrMsgTxt("Need single input & output"); 
  
  if(!(mxGetClassID(INPUT) == mxUINT8_CLASS && !mxIsSparse(INPUT) && !mxIsComplex(INPUT)))
    mexErrMsgTxt("Need dense uint8 array as input.");

  size_t size = mxGetNumberOfElements(INPUT); 
  
  unsigned char *output = mxMalloc(size + BLOSC_MAX_OVERHEAD); 

  /* asking for shuffle with elementsize = 1 does not work normally */
  int compressed_size = blosc_compress(9, 1, 1, size, mxGetData(INPUT), output, size + BLOSC_MAX_OVERHEAD); 

  mxAssert(compressed_size > 0, "blosc_compress failed"); 
 
  OUTPUT = mxCreateNumericMatrix (compressed_size, 1, mxUINT8_CLASS, mxREAL);  


  memcpy(mxGetData(OUTPUT), output, compressed_size);

  mxFree(output);  

}
