#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include <assert.h>

/* includes for mmap */
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/mman.h>



#include "flexdata.h"
#include "blosc.h"

#define NEWA(type,n) (type*)malloc(sizeof(type)*(n))
#define NEWAC(type,n) (type*)calloc(n,sizeof(type))
#define NEW(type) NEWA(type,1)

#if 0
int flexdata_sample_descriptors_xx(flexdata_t * f,
                                 int nsample,
                                 int from_bb,
                                 const int *imind,
                                 const int *bbind,
                                 float *npts,
                                 uint8 *pqcodes) {
  uint64 chani, nsq = f->nsq, nchan = f->nchannel;
  int i, ret; 
  uint8 * decoded = NULL; 
  for(chani = 0; chani < nchan; chani++) {
    flexdata_channel_t *ch = f->channels + chani;
    flexdata_block_t *b; 
    if(ch->im) b = ch->im; 
    else       b = from_bb ? ch->bb : ch->gt; 
    int imi = -1;

    /* current writing locations */
    float *d_npts = npts + chani; 
    uint8 *d_pqcodes = pqcodes + chani * nsq; 
    
    for(i = 0; i < nsample; i++) {
      if(imind[i] != imi) { /* decode for current image */
        imi = imind[i]; 
        if(!(imi >= 0 && imi < f->nimage)) goto fail;
        free(decoded); 
        decoded = NEWA(uint8, b->nbox[imi] * b->nsq); 
        ret = blosc_decompress(b->codes + b->blosc_offsets[imi], decoded, b->nbox[imi] * b->nsq);
        if(ret <= 0) 
          goto fail;
      } 
      
      if(ch->im) {
        *d_npts = b->npts[imi]; 
        memcpy(d_pqcodes, decoded, nsq);
      } else {
        if(!(bbind[i] >= 0 && bbind[i] < b->nbox[imi])) goto fail;
        *d_npts = b->npts[b->cum_nbox[imi] + bbind[i]]; 
        memcpy(d_pqcodes, decoded + bbind[i] * nsq, nsq);
      }        
      d_npts += nchan; 
      d_pqcodes += nchan * nsq;
    }
  }
  free(decoded);
  return 1; 
 fail: 
  free(decoded); 
  return 0;
}
#endif

int flexdata_sample_descriptors(flexdata_t * f,
                                 int nsample,
                                 int from_bb,
                                 const int *imind,
                                 const int *bbind,
                                 float *npts,
                                 uint8 *pqcodes) {
  if(nsample == 0) return 1; 
  int ret = 1;
  int *begins = NEWA(int, nsample + 1); 
  int nimg = 1, cur_im = imind[0], i;   
  begins[0] = 0; 
  for(i = 0; i < nsample; i++) {
    if(imind[i] != cur_im) {
      cur_im = imind[i]; 
      begins[nimg] = i; 
      nimg++;             
    }
  }
  begins[nimg] = nsample; /* sentinel */
  int nchan = f->nchannel;    

#pragma omp parallel for
  for(i = 0; i < nimg; i++) {
    int bim = begins[i], eim = begins[i + 1];
    int imi = imind[bim];
    if(!(imi >= 0 && imi < f->nimage)) {ret = 0; goto fail; }
    float *i_npts = npts + bim * (long)nchan; 
    uint8 *i_pqcodes = pqcodes + bim * (long)f->tot_nsq; 
    
    uint64 chani; 

    for(chani = 0; chani < nchan; chani++) {
      flexdata_channel_t *ch = f->channels + chani;
      flexdata_block_t *b; 
      if(ch->im) b = ch->im; 
      else       b = from_bb ? ch->bb : ch->gt; 
      int nsq = b->nsq, j;
      
      /* current writing locations */
      float *d_npts = i_npts;
      uint8 *d_pqcodes = i_pqcodes; 
      
      /* decode for current image */
      int retx; 
      uint8 *decoded;
      decoded = NEWA(uint8, b->nbox[imi] * b->nsq); 
      retx = blosc_decompress(b->codes + b->blosc_offsets[imi], decoded, b->nbox[imi] * nsq);

      if(retx != b->nbox[imi] * nsq) {ret = 0; goto fail; }
          
      if(ch->im) {
        for(j = bim; j < eim; j++) {
	  *d_npts = b->npts[imi]; 
	  memcpy(d_pqcodes, decoded, nsq);
          d_npts += nchan; 
          d_pqcodes += f->tot_nsq;
        }        

      } else {
        for(j = bim; j < eim; j++) {
          int bi = bbind[j];
          if(!(bi >= 0 && bi < b->nbox[imi])) {ret = 0; goto fail; }
          *d_npts = b->npts[b->cum_nbox[imi] + bi]; 
          memcpy(d_pqcodes, decoded + bi * (long)nsq, nsq);          
          d_npts += nchan; 
          d_pqcodes += f->tot_nsq;
        }        
      }
      /* next channel */
      i_npts += 1; 
      i_pqcodes += nsq;
      free(decoded); 
    }
  fail:;
  }
  
  free(begins); 

  return ret;
}


int flexdata_count_nbox(int nimage,
                        const uint64 * nboxes,                         
                        int ni,
                        const int *imind) {
  
  int nbox = 0, i;
  for(i = 0; i < ni; i++) {
    if(imind[i] < 0 || imind[i] >= nimage) return -1; 
    nbox += nboxes[imind[i]]; 
  }
  return nbox; 
}
                          

static void flexdata_block_accumulate_2(const flexdata_block_t *fb, 
                                 int imno,
                                 const double *af, double *a, 
                                 const double *bf, double *b) {
  int q, j, nsq = fb->nsq;
  uint64 nbox = fb->nbox[imno]; 
  /* get the codes for this image & channel */
  uint8 *pqcodes; 
  int ret;
  

  pqcodes = NEWA(uint8, nbox * nsq); 
  ret = blosc_decompress(fb->codes + fb->blosc_offsets[imno], pqcodes, nbox * nsq);
  assert(ret == nbox * nsq); 
  
  if(0) { /* non-blocked version */
    const uint8 *ci = pqcodes; 
    for(j = 0; j < nbox; j++) {
      double as = 0, bs = 0; 
      const double *aff = af, *bff = bf;
      for(q = 0; q < nsq; q++) {
        as += aff[*ci];
        bs += bff[*ci];
        ci ++;
        aff += 256; 
        bff += 256; 
      }
      a[j] = as; 
      b[j] = bs;
    }
  } else { /* with blocks so that code table remains in L1 cache */

    uint64 qb, qe;
    uint64 qbs = 16; /* qbs * 256 * sizeof(double) < L1 size */ 
    
    for(qb = 0; qb < nsq; qb += qbs) { 
      qe = qb + qbs; 
      if(qe > nsq) qe = nsq;
      
      for(j = 0; j < nbox; j++) {
        double as = 0, bs = 0; 
        const double *aff = af + qb * 256, *bff = bf + qb * 256;
        const uint8 *ci = pqcodes + j * nsq + qb; 

        for(q = qb; q < qe; q++) {
          as += aff[*ci];
          bs += bff[*ci];
          ci ++;
          aff += 256; 
          bff += 256; 
        }
        a[j] += as; 
        b[j] += bs;
      }      
    }

  }
  
  free(pqcodes); 
}

#define MAX(a, b) ((a) > (b) ? (a) : (b))

/* compute scores for all bb's on a set of images */
int flexdata_compute_bb_scores(const flexdata_t * f,
                                int nimage,
                                const int *imind,
                                const double *alls_pq,
                                const double *l2normsq,
                                double *scores, 
                                double *npts_stats) {
  int i;
  int ret = 1; 

  /* cumulated nb of boxes per image in imind */
  uint64 * cum_nbox = NEWA(uint64, nimage + 1);   
  const uint64 *n_im_box = f->nbox;
  cum_nbox[0] = 0;
  for(i = 0; i < nimage; i++) 
    cum_nbox[i + 1] = cum_nbox[i] + n_im_box[imind[i]]; 
  uint64 nboxtot = cum_nbox[nimage];

  memset(scores, 0, sizeof(scores[0]) * nboxtot);
  memset(npts_stats, 0, sizeof(scores[0]) * nboxtot * 2);
  double *sumofnpts = npts_stats; 
  double *maxofnpts = npts_stats + nboxtot;

  

#pragma omp parallel for
  for(i = 0; i < nimage; i++) {
    int imi = imind[i], j, chani; 
    int nbox0 = cum_nbox[i + 1] - cum_nbox[i];

    if(imi < 0 || imi >= f->nimage) {
      fprintf(stderr, "bad image number %d\n", imi); 
      ret = 0; 
      continue;
    }      

    const double *pq_chan = alls_pq;  /* pq coefficients for current channel */
    const double *l2sq_chan = l2normsq; /* L2 coefficients for current channel */

    double *scoresi = scores + cum_nbox[i];
    double *sumofnptsi = sumofnpts + cum_nbox[i];
    double *maxofnptsi = maxofnpts + cum_nbox[i];

    for(chani = 0; chani < f->nchannel; chani++) {
      flexdata_channel_t *ch = f->channels + chani;
      flexdata_block_t *b = ch->im ? ch->im : ch->bb;
      
      int nbox = b->nbox[imi];                 /* not same as nbox0 for im */
      double *accus;
      accus = NEWAC(double, 2 * nbox); /* accumulators */
      double *tmp4bb = accus;                  /* per-box classification score */
      double *l2sq_cell = accus + nbox;        /* per-box normalization */

      flexdata_block_accumulate_2(b, imi, 
                                  pq_chan, tmp4bb, 
                                  l2sq_chan, l2sq_cell); 
      double EPS = 1e-16;
      if(ch->bb) {
        
        const float *npts = b->npts + b->cum_nbox[imi]; 
        for(j = 0; j < nbox; j++) {
          scoresi[j] += tmp4bb[j] / MAX(EPS, sqrt(l2sq_cell[j]));
          sumofnptsi[j] += npts[j]; 
          if(npts[j] > maxofnptsi[j]) maxofnptsi[j] = npts[j]; 
        }

      } else {
        /* copy over all scores */
        double score = tmp4bb[0] / MAX(EPS, sqrt(l2sq_cell[0]));
        for(j = 0; j < nbox0; j++)
          scoresi[j] += score; 
      }
      free(accus);
      
      pq_chan += 256 * b->nsq;
      l2sq_chan += 256 * b->nsq; 
    } 
  }
  free(cum_nbox); 
  return ret;
}


/************************************************************ 
 * Reading flexdata files
 ************************************************************/



/* load header in flexdata info format and compare with reference header */

static int load_header(const char * fname, const uint64 *ref_header, int *nimage_out, uint64 **code_sizes) {
  uint64 header[5]; 
  
  FILE *f = fopen(fname, "r"); 
  if(!f) {
    fprintf(stderr, "cannot open %s\n", fname); 
    return 0; 
  }
  fread(header, sizeof(uint64), 5, f); 
  if(memcmp(header, ref_header, sizeof(uint64) * 4)) {
    fprintf(stderr, "inconsistent %s\n", fname); 
    return 0;     
  }
  int nimage = header[4]; 
  *nimage_out = nimage; 
  *code_sizes = NEWA(uint64, nimage * 2); 
  fread(*code_sizes, nimage * 2, sizeof(uint64), f); 
  fclose(f); 
  return 1;
}
  
  
  



#define FOPEN(pat) \
  sprintf(fname, pat, dirname, fieldname);  \
  f = fopen(fname, "r");  \
  if(!f) {\
    fprintf(stderr, "cannot open %s\n", fname); \
    goto fail;\
  }

 
flexdata_block_t *flexdata_block_new(const char *dirname, const char *fieldname, int do_mmap) {
  
  fprintf(stderr, "  loading flexdata block %s %s, mmap=%d\n", dirname, fieldname, do_mmap); 

  flexdata_block_t *b = NEWAC(flexdata_block_t, 1); 

  uint64 ref_header_npts[4] = {1, 2, 4, 2}; 
  uint64 ref_header_codes[4] = {1, 0, 1, 2};   
  uint64 *npts_sizes = NULL, *code_sizes = NULL; 
  
  int nimage2, i; 
  /* load headers */

  char fname[1000]; 
  sprintf(fname, "%s/npts_%s.info", dirname, fieldname); 
  int ret = load_header(fname, ref_header_npts, &b->nimage, &npts_sizes); 
  
  if(!ret) goto fail; 

  sprintf(fname, "%s/pq_%s.info", dirname, fieldname); 
  ret = load_header(fname, ref_header_codes, &nimage2, &code_sizes); 

  if(!ret) goto fail;
  if(nimage2 != b->nimage) {
    fprintf(stderr, "inconsistent nb of images in npts and pq files (%d %d)\n", b->nimage, nimage2);
    goto fail;
  }
  b->nbox = NEWA(uint64, b->nimage); 
  b->cum_nbox = NEWA(uint64, b->nimage + 1); 
  b->cum_nbox[0] = 0;
  b->nsq = code_sizes[0];

  for(i = 0; i < b->nimage; i++) {
    if(npts_sizes[2 * i + 1] != code_sizes[2 * i + 1] || 
       npts_sizes[2 * i] != 1 || code_sizes[2 * i] != b->nsq) {
      fprintf(stderr, "inconsistent sizes in flexdata block"); 
      goto fail; 
    }
    b->totbox += b->nbox[i] = npts_sizes[2 * i + 1]; 
    b->cum_nbox[i + 1] = b->cum_nbox[i] + b->nbox[i];
  }
 
  FILE *f;
  
  FOPEN("%s/pq_%s.bloscinfo")
  
  b->blosc_offsets = NEWA(uint64, b->nimage + 1);   
  fread(b->blosc_offsets, b->nimage + 1, sizeof(uint64), f);   
  fclose(f); 
  if(b->blosc_offsets[0] != b->nimage) {
    fprintf(stderr, "inconsistent image sizes"); 
    goto fail;
  }  

  b->blosc_offsets[0] = 0; /* convert size of blosc blocks to offsets */
  for(i = 0; i < b->nimage; i++) 
    b->blosc_offsets[i + 1] += b->blosc_offsets[i]; 
  
  /* now read main data */

  if(!do_mmap) {

    b->npts = NEWA(float, b->totbox); 
    if(!b->npts) {
      fprintf(stderr, "allocation for npts failed\n"); 
      goto fail; 
    }
    FOPEN("%s/npts_%s.data")
    fread(b->npts, b->totbox, sizeof(float), f);
    fclose(f);   
    
    b->codes = NEWA(uint8, b->blosc_offsets[b->nimage]); 
    if(!b->codes) {
      fprintf(stderr, "allocation for codes failed\n"); 
      goto fail; 
    }
    FOPEN("%s/pq_%s.blosc")
    fread(b->codes, b->blosc_offsets[b->nimage], sizeof(uint8), f);
    fclose(f);   

  } else {
    sprintf(fname, "%s/npts_%s.data", dirname, fieldname);
    b->fd1 = open (fname, O_RDONLY);
    if (b->fd1 < 0) {
      perror ("open before mmap failed");
      goto fail;
    }

    /* map whole file */
    struct stat sb;
    fstat (b->fd1, &sb);
    b->sz1 = sb.st_size; 
    void *mm = mmap (NULL, sb.st_size, PROT_READ, MAP_SHARED, b->fd1, 0);
    if (mm == MAP_FAILED) {
      perror ("map failed");
      goto fail; 
    }
    b->npts = mm; 

    sprintf(fname, "%s/pq_%s.blosc", dirname, fieldname);
    b->fd2 = open (fname, O_RDONLY);
    if (b->fd2 < 0) {
      perror ("open before mmap failed");
      goto fail;
    }
    /* map whole file */
    fstat (b->fd2, &sb);
    b->sz2 = sb.st_size; 
    mm = mmap (NULL, sb.st_size, PROT_READ, MAP_SHARED, b->fd2, 0);
    if (mm == MAP_FAILED) {
      perror ("map failed");
      goto fail; 
    }
    b->codes = mm; 
  }
  return b;   
 fail: 
  flexdata_block_delete(b);
  return NULL;    
}

  


void flexdata_block_delete(flexdata_block_t *b) {
  if(!b) return; 

  free(b->nbox); 
  free(b->cum_nbox); 
  if(b->fd1 > 0) {
    munmap (b->npts, b->sz1);
    close (b->fd1);
  } else 
    free(b->npts); 
  free(b->blosc_offsets); 
  if(b->fd2 > 0) {
    munmap (b->codes, b->sz2);
    close (b->fd2);
  } else     
    free(b->codes); 
  free(b); 
}


#define CHECKSIZES(b)                                                   \
      if(!b) goto fail;                                                 \
      if(f->nimage < 0) {                                               \
        f->nimage = b->nimage;                                          \
      } else if(f->nimage != b->nimage) {           \
        fprintf(stderr, "inconsistent sizes across channels!\n");       \
        goto fail;                                                      \
      }


flexdata_t *flexdata_new(int nchannel, 
                         const char **dirnames, 
                         const int *immasks, 
                         int do_mmap) {
  flexdata_t *f = NEWAC(flexdata_t, 1); 
  f->nchannel = nchannel; 
  f->channels = NEWAC(flexdata_channel_t, nchannel); 
  f->nimage = -1; 
  f->tot_nsq = 0;
  int i; 
  for(i = 0; i < nchannel; i++) {
    flexdata_channel_t *ch = f->channels + i; 
    if(immasks[i]) {
      ch->im = flexdata_block_new(dirnames[i], "im", 0); 
      CHECKSIZES(ch->im); 
      if(ch->im->totbox != f->nimage) {
        fprintf(stderr, "should be 1 descriptor per image\n"); 
        goto fail;
      }
      f->tot_nsq += ch->im->nsq; 
    } else {
      ch->gt = flexdata_block_new(dirnames[i], "gt", do_mmap); 
      CHECKSIZES(ch->gt); 
      ch->bb = flexdata_block_new(dirnames[i], "bb", do_mmap); 
      CHECKSIZES(ch->bb);
      if(!f->nbox) {
        f->nbox = ch->bb->nbox; 
        f->cum_nbox = ch->bb->cum_nbox; 
      }       
      f->tot_nsq += ch->bb->nsq; 
    }
  }  
 

  return f; 
 fail: 
  flexdata_delete(f); 
  return NULL; 
}
                         
void flexdata_delete(flexdata_t *f) {
  if(!f) return; 
  int i; 
  for(i = 0; i < f->nchannel; i++) {
    flexdata_channel_t *ch = f->channels + i; 
    flexdata_block_delete(ch->im); 
    flexdata_block_delete(ch->bb); 
    flexdata_block_delete(ch->gt); 
  }
  free(f->channels);
  free(f);   
}
