#ifndef FLEXDATA_CLIENT_SERVER_INCLUDED
#define FLEXDATA_CLIENT_SERVER_INCLUDED

#include "flexdata.h"

/* boilerplace socket based. All servers map the whole datafiles, but
   the client sends requests only to some of the servers. Relies on
   mmap to keep memory manageable. */


typedef struct {
  /* must be set on input of flexdata_client_new */
  char name[256]; /* host name */
  int port;       /* TCP port */
  int last_image; /* index of last image handled by this server */

  /* filled in by flexdata_client_new */
  int sock;       /* socket */
} flexdata_server_t; 



typedef struct {
  int nserver; 
  flexdata_server_t *servers; 

  int *im_to_serv; /* index of server to which each image will be directed */

  /* similar to normal flexdata structure */
  int nimage; 
  int tot_nsq; 
  int nchannel; 
  uint64 *nbox;       
  uint64 *cum_nbox;
} flexdata_client_t; 


flexdata_client_t *flexdata_client_new(int nserver, 
                        flexdata_server_t *servers, /* copied, fill in the 3 first fields */
                        int nchannel, 
                        const char **dirnames, 
                        const int *immmasks); 

void flexdata_client_delete(flexdata_client_t *fc); 



int flexdata_client_sample_descriptors(flexdata_client_t * fd,
                                       int nsample,
                                       int from_bb,
                                       const int *imind,
                                       const int *bbind,
                                       float *npts,
                                       uint8 *pqcodes);


int flexdata_client_compute_bb_scores(const flexdata_client_t * fd,
                                      int nimage,               /* nb of images we want scores for */
                                      const int *imind,         /* indices of images we want to handle */
                                      const double *alls_pq,    /* coefficients for numerators */
                                      const double *l2normsq,   /* coefficients for per-cell normalization */
                                      double *scores,           /* output scores, size (nb of bboxes per image) */
                                      double *npts_stats);      /* per-box statistics on nb of points */






#endif
