function [MoFA A]= init_mfa(X,C,ops)

% initialize from a few kmeans iterations, and set variances using PPCA model

[D N] = size(X);

MoFA.mix   = ones(C,1) / C;
MoFA.Psi   = zeros(D,C);
MoFA.W     = zeros(D,ops.dim,C);

iter = 0;
MaxIters = 100;
while iter < MaxIters; % try at most MaxIters kmeans runs to find clusters of size at least ops.dim
  iter       = iter+1;
  [A,MoFA.M] = kmeans(X,C,ops.ini,0,ops.maxbyte); % diagnosis=0
  Count      = accumarray(A,1,[C, 1]);
  if ops.dia
    fprintf('--> K-means run %d to find clusters of size >= %d, yielded %d in smallest cluster \n',iter,ops.dim+2, min(Count));
  end
  if min(Count) >= (ops.dim+2); iter=MaxIters+1;end
end

done       = find(Count > (ops.dim+2)); % identify clusters with d points or less

for c = done(:)'  
  cv = cov(X(:,A==c)');
  v  = diag(cv);    
  
  if ops.dim > 0    
    [a,b] = eigs(cv,[],ops.dim,'lm');    
    b = diag(b);    
    sigma = (sum(v) - sum(b) ) / (D-ops.dim); % average of variance outside subspace
    MoFA.W(:,:,c) = a*diag(sqrt(b-sigma));   % factor loadings with variance inside subspace     
    MoFA.Psi(:,c) = sigma*ones(D,1); % noise variance outside subspace equal in all dimensions, PPCA solution
  else
    MoFA.Psi(:,c) = v; % cluster variances
  end
end

if any(~done)
  MoFA.Psi(:,~done) = mean(MoFA.Psi(:,done),2);  % initialize variances as average of non-degenerate components
  MoFA.W(:,:,~done) = mean(MoFA.W(:,:,done),3);  % initialize subspace vectors as average of non-degenerate components, anything better ?!
end

% GOKBERK UPDATE
% MoFA.Psi = max(MoFA.Psi,ops.minvar); % Aug'11
MoFA.Psi = bsxfun(@max,MoFA.Psi,ops.minvar(:)); % Oct'11

if ops.iso
  MoFA.Psi = ones(D,1)*mean(MoFA.Psi,1);
end
if ops.eqn
  MoFA.Psi = (MoFA.Psi*MoFA.mix)*ones(1,C);
end
