function [ LogL, proj,lat_covs ] = ll_mfa(X,mfa,X2)
%
% function [ LogL, proj ] = mppca_gauss(X,M,W,Psi);
%
% computes log-likelihoods of X under Gaussians
% also returns the inferred latent coordinates
%
% X (D x N)     - n d-dimensional data
% mfa is a struct with the fields:
% M (D x C)     - k d-dimensional means
% W (D x d x C) - for each Gaussian q d-dim vectors
% Psi (D,C)     - diagonal noise levels

N         = size(X,2);
[D, d, C] = size(mfa.W);

if nargin==2; X2 = X.^2;end

if ~isfield(mfa,'psiI')
    psiI  = mfa.Psi.^-1;
else
    psiI = mfa.psiI;
end

if ~isfield(mfa,'psiIM')
    psiIM  = psiI .* mfa.M;
else
    psiIM = mfa.psiIM;
end

if ~isfield(mfa,'sum_psiIM2')
    sum_psiIM2 = sum(psiI.*(mfa.M.^2) , 1);
else
    sum_psiIM2 = mfa.sum_psiIM2;
end

LogL  = (-0.5*psiI)'*X2;
clear X2;
LogL  = LogL  + (psiIM)'*X;
tmp   = sum_psiIM2 - sum(log(psiI),1)+ D*log(2*pi);
LogL  = bsxfun(@minus,LogL,0.5*tmp');

% d=0 for diagonal covariances

if d>0        

    if isfield(mfa,'latprep')
        latprep = mfa.latprep;
    else
        latprep = mfa_preplatcase(setfield(mfa,'psiI',psiI));
    end

    psiI_w_xc    = latprep.psiIw * X; % (d*K N) SLOW
    psiI_w_xc    = bsxfun(@minus,psiI_w_xc,latprep.psiIwm);

    proj = latprep.LatCovPsiIw*X; % S * W * inv(Psi) * X     (d*K N)  SLOW
    proj = bsxfun(@minus,proj,latprep.LatCovPsiIwm);  % S * W * inv(Psi) * ( X - mu )  -- (d*K N)
    % proj: (d*C D)(D N) -> (d*C N), can be reshaped into (d C N)

    % psiI_w_xc .* proj
    x = detach(psiI_w_xc);
    x = x .* proj;

    LogL = LogL + latprep.accumatrix*x;
    LogL = bsxfun(@plus,LogL,latprep.LogDet/2);

    lat_covs = latprep.lat_covs;

elseif nargout > 1

    lat_covs = zeros(d,d,C);
    proj = zeros(C*d,N);

end



