function [L,mfa] = mfa_EM_chunked(ops,X,mfa,X2,splitind); % E-step
% Chunked E&M steps together. 

N       = size(X,2);
[D,d,C] = size(mfa.W);

sumQ = []; % (1 K)

% prepare for ll_mfa --- these should be deleted at the end of the function!
mfa.psiI  = mfa.Psi.^-1;
mfa.psiIM  = mfa.psiI .* mfa.M;
mfa.sum_psiIM2 = sum(mfa.psiI.*(mfa.M.^2) , 1);
if d>0
    mfa.latprep = mfa_preplatcase(mfa);
end

% init accumulated stats
if d==0
    XQ = [];
    X2Q = [];
elseif ops.iso
    M = [];
    S = [];
end

L = 0;
for i = length(splitind):-1:1

    clear cX cX2 cQ
    jj = splitind{i};
    cX = X(:,jj);
    cX2 = X2(:,jj);

    [cQ, cL] = mfa_E_step(cX,mfa,cX2,setfield(ops,'timing',false));
    L = L + detach(cL)*size(cX,2)/N;
    sumQ = setorplus(sumQ,sum(cQ,2)');

    if d==0
        % We use the fact that [X1 X2]*[Q1 Q2]'=X1*Q1+X2*Q2
        XQ = setorplus(XQ,cX*cQ');
        X2Q = setorplus(X2Q,cX2*cQ');
    elseif ops.iso

        xp.X = cX.'; % transpose should be better.
        xp.shapeX = 'NxD';
        xp.W = cQ;
        xp.shapeW = 'KxN';
        xp.center = 0;
        xp.normw = 0;
        [xS,xM] = cn_covW(xp);
        assert(isequal(size(xM),[D C]));

        S = setorplus(S,xS);
        M = setorplus(M,xM);

        clear xp xS xM
    else
        error('chunked version is not implemented yet');
    end

end

clear mfa; % below, mfa is not necessary.

if d==0
    mfa = helperM_diag(X,X2,sumQ,ops,XQ,X2Q,d,C);
    % no need to rmfield psiI and M2, helperM_diag creates mfa from sratch.
elseif ops.iso


    x = sumQ; x(x==0)=1; nn = 1./x; % (1 C)
    M = bsxfun(@times,M,nn);
    S = bsxfun(@times,S,reshape(nn,1,1,C));
    M3 = reshape(M,[],1,C);  % (D 1 C)
    S = S - mtimesx(M3,M3,'T','matlab');

    mfa = mfa_M_ppca(sumQ,M,S,D,d,C);

    clear M S M3
else
    error('chunked version is not implemented yet');
end

mfa.Psi = bsxfun(@max,mfa.Psi,ops.minvar(:)); % Oct'11

if ops.eqn
    mfa.Psi = (mfa.Psi*mfa.mix)*ones(1,C);
end

mfa = rmfieldp(mfa,{'psiI','psiIM','sum_psiIM2','latprep'});








function mfa = helperM_diag(X,X2,sumQ,ops,XQ,X2Q,d,C)

[D N]   = size(X);

%sumQ    = sum(Q,2)'; % (K N) -> (1 K)
mfa.mix = sumQ' / sum(sumQ);
mfa.W   = zeros(D,d,C);
%XQ      = X *Q'; % (D N)(K N)' -> (D K)
%X2Q     = X2*Q'; 


if any(isnan(mfa.mix)); keyboard;end

assert(d==0);

mfa.M = bsxfun(@times, XQ, sumQ.^-1);
mfa.Psi = bsxfun(@times, X2Q, sumQ.^-1);
mfa.Psi = bsxfun(@minus, mfa.Psi, mfa.M.^2);  

if ops.iso
    mfa.Psi = ones(D,1)*mean(mfa.Psi,1);
end

