function	[Q, L, Lats, lat_covs] = mfa_E_step(X,mfa,X2,ops)
%
% Q     (K N)
% L     (K N)
% Lats  (C*d N) (projections)
%
% ops
%   .timing
%   .sparsityk

N         = size(X,2);
[D, d, C] = size(mfa.W);

tt = tic;
if ~exist('X2','var')
    [L, Lats, lat_covs] = ll_mfa(X,mfa);
else
    [L, Lats, lat_covs] = ll_mfa(X,mfa,X2);
end
if ops.timing
    fprintf('ll_mfa: %.2f secs\n',toc(tt));
end
% Gokberk: L -> log p(x,z)

tt = tic;

L  = bsxfun(@plus,L,log(mfa.mix+realmin)); % Gokberk: L -> log p(x,z) (K N)

if isnan(L); keyboard;end

Q    = bsxfun(@minus,L,max(L,[],1)); % subtract largest energy
Q    = exp(Q)+realmin;               % exponentiate
Q    = bsxfun(@times,Q,sum(Q,1).^-1);% normalize
% Q -> p(z|x) (C N) each col sums to 1

Q = cn_sparsifypbdist(Q,'KxN',N,C,ops.sparsityk);
if issparse(Q)
    fprintf('Q sparse nnz/numel: %d\n',nnz(Q)/numel(Q));
else
    disp('Q is dense');
end

if any(isnan(Q(:))); keyboard;end

if (nargout > 1)    
    % Gokberk: instead of this, we could use mean(L(1,:) - log(Q(1,:))) to get average log p(x) here.
    % BUT 
    % 1) That would fail at points where Q is small. Here such cases are removed.
    % 2) Below code is the free energy (not just approx loglikehood) when approximate sparse posteriors are used 
    % -- ie variational EM. Therefore, this is good for variational EM.
    L = L-log(Q+realmin); % Gokberk: L -> log p(x) --> each column contains the replicate of the same numbers.
    L = L(:)'*Q(:);  %  sum_x sum_z p(z|x) log p(x) = sum_x log p(x) 
    L = L / size(X,2); % report average log-likelihood. average log p(x).
end

if ops.timing
    fprintf('mfa_E_step (excluding ll_mfa): %.2f secs\n',toc(tt));
end


