
% extract Fisher Vectors on an image from a set of bounding boxes + masks (if requested)
% adapted from Gokberk's fv_objdet_fvmog_exactfv

function [per_cell_descs, per_cell_npts] = fv_from_samples(img_filename, imsize, desc, bbs, PCA, dict, grids, masks)

  
  ncell = sum(prod(vertcat(grids{:}),2)); % number of spatial cells
  nobj = size(bbs, 1);   
    
  masked = ''; 
  if ~isempty(masks) 
    masked = 'using masks'; 
  end
  
  fprintf('    extracting %d FVs on %d grid cells from image %s %s\n', nobj, ncell, img_filename, masked);
  
  cn_setrandseed(101); 
    
  o_rsz = 0; 
  o_nsc = Inf;
  f_scstep = 1.2; % should be in sync with extract_features pfeat.scstep
  f_sz = 12;      % same, pfeat.sz
  o_scindexstep = 1; 
    
        
  [f_xynorm, d_cols] = extract_features(img_filename, desc, true);

  f_nsc = length(f_xynorm);
  posts = {};
    
  % convert f_xynorm to int16 and apply PCA to local descriptors
  for sci = 1:f_nsc
    f_xynorm{sci} = int16(bsxfun(@times, f_xynorm{sci}, [imsize(2); imsize(1)] - 1) + 1 );
    d_cols{sci} = pcaApply(d_cols{sci}, PCA.U, PCA.mu, PCA.ndim);      
    x = cn_gaussdiag_pb( d_cols{sci}, dict, {'post'}); 
    post = x.post;
    % keep 1 assignment per Gaussian
    post = cn_sparsifypbdist(post, 'NxK', [], dict.K, 1);       
    posts{sci} = post;
  end
    
  % use height as reference size of samples
  obj_sc = double(bbs(:,4)) - double(bbs(:,2)) + 1;

  % M12 not sure this is useful when o_rsz and o_nsz are 0 and Inf
  minscs = min(max(round( reallog( obj_sc .* (o_rsz ./ f_sz) ) / reallog(f_scstep) + 1 ),1),f_nsc); % +1 to convert into matlab coordinates
  maxscs = min(minscs + (o_nsc-1) * o_scindexstep, f_nsc); 
  scstep = o_scindexstep; 
    
  % get spatial grid cells per object bounding box (4 nobj ncell)
  bbsTD = double(bbs.'); % (4 nobj)
  all_bbsT = permute( int16(round(cn_spatialgrid(bbsTD, grids,...
                                                 struct('normcoor',false,'fixcoor',false)))), [1 3 2] ); % (4 nobj ncell)
    
  % from cn_fvgmm_poolobjdesc

  per_cell_descs = {};
  per_cell_npts = {};
  
  for celli = ncell:-1:1
  
    % prepare arguments for Mex function (convert to 0-based and change some names)
    p = struct();
    p.f_xy = {};
    for sci = 1:f_nsc
      p.f_xy{sci} = f_xynorm{sci} - 1 ;
    end
    p.bbs  = all_bbsT(:,:,celli) - 1 ;
    p.posts = posts; 
    p.minscs = int16(minscs - 1); 
    p.maxscs = int16(maxscs - 1); 
    p.d_cols = d_cols; 
    p.mix = dict.mix; 
    p.vars = dict.vars;
    p.means = dict.mu; 
    
    p.op = 'fvboxes';
    p.desctrans = 'avg'; 
    p.scstep = scstep;     

    if ~isempty(masks)
      % crop mask to current cell
      xmasks = masks; % init
      xbbsTD = double( all_bbsT(:,:,celli) );
      assert( size(xbbsTD,2) == length(masks) );
      for i = 1:length(masks) % over objects
        xrel = round(xbbsTD(:,i) - bbsTD([1;2;1;2],i) + 1);
        xmasks{i} = xmasks{i}( xrel(2):xrel(4), xrel(1):xrel(3) );
        assert( size(xmasks{i},1) == (xbbsTD(4,i)-xbbsTD(2,i)+1) );
        assert( size(xmasks{i},2) == (xbbsTD(3,i)-xbbsTD(1,i)+1) );
      end
      p.masks = xmasks;
    end

    [D, K] = size(dict.mu);
    %K = size(out,2);
    %R = length(npts);

    if nobj > 0
      % compute FVs for the windows, and maybe the masks
      [xdesc, xcellnpts] = cn_fvgmm_poolobjdesc_mex(p);

      xdesc = cn_fvgmm_reshape('kmajor->stacked', xdesc, D, K);
    else      
      xdesc = zeros(D * K * 2 + K, 0); 
      xcellnpts = zeros(1, 0); 
    end  
    
    
    assert(size(xdesc,2)==nobj);
    xcellnpts = single(xcellnpts);
    
    per_cell_descs{celli} = single(xdesc); 
    per_cell_npts{celli} = xcellnpts; 
    
  end
  
  
end
  



