function pq = pq_prep(pq,opt)
% pq = pq_prep(pq)
%
% Do additional precomputations needed for my implementation of pq_assign()
% and other new functions like pq_pool*()
%
% A good practice is to store pq_new() output on the disk and apply this function
% only once just after loading the model.
% 
% Usage: pq = pq_prep(pq,opt)
%
% INPUT
% pq                Original PQ structure,supports an array of pq structs.
% [opt]             Cell array of optional fields to add, a subset of {'l2normsq'}   
%
% Adds the following fields:
% pq.cent3s         (ds ks nsq) single 
%   .[l2normsq]     (ks nsq)    double  Squared L2 norm of each center subvector.
% 
% R.G.Cinbis Jan 2012

if nargin==1
    opt = {};
end

for c = 1:length(pq)
    pq2{c} = pq_prep2(pq(c),opt);
end
pq = [pq2{:}];



function pq = pq_prep2(pq,opt)

pq.cent3s = cat(3,pq.centroids{:});
assert(isequal( cn_sizend(pq.cent3s,3), [ pq.ds, pq.ks, pq.nsq ] ));
assert(isa( pq.cent3s, 'single' ));

assert(iscellstr(opt));
for i = 1:length(opt)
    switch(opt{i})
        case 'l2normsq'
            pq.l2normsq = reshape(sum(double(pq.cent3s).^2,1),256,pq.nsq); % (ks nsq) -- for L2 normalization
        otherwise
            error('unknown');
    end
end

