#include "poolobjdesc_init.hpp"

/**
 * Author: R. Gokberk Cinbis.
 */


MixOfGauss<double> spatgmm( D, K, shared_ptr<const double>( mix, DeleterNone<const double> ), shared_ptr<const double>( means, DeleterNone<const double> ), shared_ptr<const double>( vars, DeleterNone<const double> ) );

if ( !fvnormalizer )
    throw runtime_error("dictionary is required for spatial fisher vectors");

// K and Kapp may differ.
// Dapp is not important since we ignore it here.
if (D != 2) // based on dictionary
    throw runtime_error("D=2 is required for spatial fisher vectors");

mwSize maxN = max(allN) ;
mwSize maxthread = setnumparallel( R );
mwSize nscale = getnscale();
if (nscale == 0)
    throw runtime_error("nscale cannot be 0");

// we cannot precompute descriptors. we'll get them on the fly.
vector< shared_ptr<double> > allTmpLocs;
vector< shared_ptr<double> > allTmpPosts;
mwSize D2 = D * 2;
allTmpLocs.reserve( maxthread );
allTmpPosts.reserve( maxthread );
for (mwIndex i = 0; i < maxthread; ++i ) {
    allTmpLocs.push_back( shared_ptr<double>(new double[ D2 * maxN ], DeleterArray<double>) ); // to store normalized coordinates and squares of points in a scale.
    allTmpPosts.push_back( shared_ptr<double>(new double[ K * maxN ], DeleterArray<double>) ); // to store posterior wrt spatial dictionary of a single point.
}

shared_ptr<mask_t> tmpmasks(new mask_t[ maxN * maxthread ],DeleterArray<mask_t>);
string errstr;

#pragma omp parallel for
for (mwSize r = 0; r < R; ++r) {
    try {

#ifdef POOL_TREE
        if ( skipifnonexist && (!segmentexists.get()[r]) )
            continue; 
#endif

        const mwSize threadi = getthreadindex(maxthread);

        mask_t *tmpm = tmpmasks.get() + threadi * maxN;
        npts_t obj_npts = 0; // number of points within the object
        double * obj_out = out + (1 + 2*D) * K * Kapp * r;

        int16_t minscale,maxscale;
        getMinMaxScaleForRegion( r, &minscale, &maxscale );
        for (int16_t sci = minscale; sci <= maxscale; sci+=scstep) {

            const double *Q = allQ[sci];
            const mwIndex *Q_Ir = allQ_Ir[sci];
            const mwIndex *Q_Jc = allQ_Jc[sci];

            // it is definite that we'll compare each point wrt current box at least once.
            // so it is better check beforehand which points are inside the box.
#if defined(POOL_BOXES) || defined(POOL_BOXES_WITH_PTWEIGHTS)
            // if setmasks() we may effectively be pooling segments here as well.  however, it is more flexible but
            // less efficient here as we allow overlapping segments given as per-box segmentation masks.
            obj_npts += getptmask4box(sci,r,tmpm);
#elif defined(POOL_TREE)
            // convert compact "points within the segment" list into a mask of size (1 curnpts), which is far
            // easier to handle below
            obj_npts += getptmask4segment(nscale,sci,r,tmpm);
#else
#error unknown
#endif

            double *V = allTmpLocs[threadi].get(); // will generate descriptors on the fly
            double *Qspat = allTmpPosts[threadi].get(); // (K Npt)

            // --old--
            // const Tc * obj_bb = bbs.get() + 4 * r;
            // const mwSize &N = allN[sci];
            // const Tc *f_xy = allLocs[sci].get();
            // getNormLocsWrtObject( sci, r, tmpm, V );
            PoolObjDesc_Private::getNormLocsWrtObject( bbs.get() + 4 * r, allN[sci], allLocs[sci].get(), tmpm, V );
            PoolObjDesc_Private::getSpatialPosteriors( allN[sci], V, spatgmm, tmpm, Qspat ); // (K Npt)

            double *obj_out_mat = obj_out;
            for (mwSize kapp = 0; kapp < Kapp; ++kapp) {

                const double *xQ = Q + Q_Jc[kapp]; 
                mwSize xnnz = Q_Jc[kapp+1] - Q_Jc[kapp]; /* #nonzero in Q(:,kapp) */
                const mwIndex *xIr = Q_Ir + Q_Jc[kapp]; 

                for ( mwSize zi = 0; zi < xnnz; zi++ ) { // loop over nonzero elements in Q's current column
                    const mwSize &pti = *xIr;


#if HAS_PT_WEIGHTS
                    if (tmpm[pti] > FvGmm_PoolObjDesc_PtWeightEps) {  // check whether point is inside the box

                        double *xout = obj_out_mat;
                        const double *qspat = Qspat + K * pti;
                        const double ptw = *xQ * tmpm[pti];

                        for (mwSize kspat = 0; kspat < K; ++kspat) { // increment out(:,kspat,kapp,:)
                            const double *xV = V + D2 * pti; // for every spatial center we use the same location descriptor!
                            double post = *(qspat++) * ptw;  // p(kapp,kspat|x,l) = p(kapp|x) * p(kspat|l)  
                            *(xout++) += post;
                            for (mwSize i = 0; i < D2; ++i) {
                                (*xout++) += (*xV++) * post; // increment p(k|x)*x and p(k|x)*(x^2)
                            }
                        }

                    }
#else
                    if (tmpm[pti]) {  // check whether point is inside the box

                        double *xout = obj_out_mat;
                        const double *qspat = Qspat + K * pti;

                        for (mwSize kspat = 0; kspat < K; ++kspat) { // increment out(:,kspat,kapp,:)
                            const double *xV = V + D2 * pti; // for every spatial center we use the same location descriptor!
                            double post = *xQ * *(qspat++);  // p(kapp,kspat|x,l) = p(kapp|x) * p(kspat|l)  
                            *(xout++) += post;
                            for (mwSize i = 0; i < D2; ++i) {
                                (*xout++) += (*xV++) * post; // increment p(k|x)*x and p(k|x)*(x^2)
                            }
                        }

                    }
#endif


                    ++xQ; ++xIr;  // next nonzero Q row
                } 
                obj_out_mat += (D2 + 1) * K; // next (:,:,kapp,r), skip the matrix of size (D2+1 Kspat)

            } // loop over k
        } // loop over sci

        npts[r] = obj_npts; // set after counting over all scales.

        // we've pooled over scales, now convert moments into fisher vectors with analytical approximation.
        // this operation can be done independently for each k
        //
        // note that we effitively use "Kapp" copies of the spatial dictionary model.
        double *obj_out_mat = obj_out;
        for (mwSize kapp = 0; kapp < Kapp; ++kapp) {
            stats2fv(obj_out_mat,obj_npts,avgfv);
            obj_out_mat += (D2 + 1) * K; // next (:,:,kapp,r), skip the matrix of size (D2+1 Kspat)
        }

    } 
#include "poolobjdesc_catcherr.hpp"
    
}

#include "poolobjdesc_end.hpp"

