
/**
 * Author: R. Gokberk Cinbis
 */



#include "mex.h"
#include <omp.h>
#include <cmath>
#include <math.h>
#include <vector>
#include <algorithm>
#include <cstring>
#include "cn_threads.hpp"
#include "cn_matlabarrays.hpp"
#include <vector>
#include <iostream>
#include <string>

using std::vector;
using std::cout;
using std::endl;
using std::string;
using std::runtime_error;

enum {a_op=0, a_data, a_D, a_K};

void mexFunction(int , mxArray *plhs[], int nrhs, const mxArray *prhs[]) {
    
    if (nrhs!=4)
        mexErrMsgTxt("invalid number of input arguments");

    const mwSize D = (mwSize) cn::mex::getscalar(prhs[a_D]);
    const mwSize K = (mwSize) cn::mex::getscalar(prhs[a_K]);
    const double * data = cn::mex::getDenseData<double>(prhs[a_data],"data");
    
    char *opstring = mxArrayToString(prhs[a_op]);
    if (opstring==NULL)
        mexErrMsgTxt("p.op is not a proper string");

    if (strcmp(opstring,"kmajor->stacked")==0) {

        if ( cn::mex::getdim(prhs[a_data],1)!=(1+D*2) ) 
            mexErrMsgTxt("size(data,1) is wrong");
        if ( cn::mex::getdim(prhs[a_data],2)!=K )
            mexErrMsgTxt("size(data,2) is wrong");
        if ( mxGetNumberOfDimensions(prhs[a_data]) > 3 )
            mexErrMsgTxt("ndims(data) is wrong");
        mwSize R = cn::mex::getdim(prhs[a_data],3);

        // alloc output
        plhs[0] = cn::mex::createArrayWoInit(mxDOUBLE_CLASS,K*(1+D*2),R);
        double * out = (double *) mxGetData( plhs[0] );

        mwSize nbD = sizeof(double) * D;
        mwSize K_DK = D*K+K;
        mwSize slice = 2*D*K+K;

#pragma omp parallel for
        for (mwIndex r = 0; r < R; ++r) {
            mwSize bias = r * slice;
            const double * from = data + bias;
            double * out_mix = out + bias;
            double * out_mu = out + bias + K;
            double * out_var = out + bias + K_DK;

            for (mwIndex k = 0; k < K; ++k) {
                *(out_mix++) = *(from++);

                memcpy( out_mu, from, nbD );
                out_mu += D;
                from += D;

                memcpy( out_var, from, nbD );
                out_var += D;
                from += D;
            }

        }


    } else {
        mexErrMsgTxt("unknown op string");
    }
    

}

