function im2 = cn_imresize(im,sz,method)
% Resize an image.
%
% INPUT
% im            (H W B) Image.
% sz            A scale factor or [height width], where either height or width can be NaN.
% method        'cvNearest'     OpenCV nearest neighbor.
%               'cvLinear'      OpenCV bilinear interpolation.
%               'cvArea'        OpenCV resampling using pixel area relation, moire-free when down-sampling.
%               'cvCubic'       Bi-cubic interpolation in 4x4 neighborhood.
%               'cvLanczos4'    Lanczos interpolation in 8x8 neighborhood.
%               TODO Fix bicubic interpolation so that it returns only correct range!
%
% OUTPUT
% im2           Resized image.
%
% For now, I think the best strategy is to 
%               
% R.G.Cinbis, Sep 2012
%
% To test methods:
% clf; 
% subplot(2,2,1); imshow(cn_imresize(im,sz,'cvNearest'));
% subplot(2,2,2); imshow(cn_imresize(im,sz,'cvArea'));
% subplot(2,2,3); imshow(cn_imresize(im,sz,'cvCubic'));
% subplot(2,2,4); imshow(imresize(im,sz,'bicubic'));

sz = double(sz);
if isscalar(sz)
    sz = [size(im,1) size(im,2)] * sz; 
end
% from now on, we can assume sz is a double-typed integer array.

m = isfinite(sz);
if ~m(1) % sz(1) is nan
    sz(1) = sz(2) / size(im,2)  * size(im,1);
elseif ~m(2)
    sz(2) = sz(1) / size(im,1)  * size(im,2);
else
    assert(all(m));
end

sz = ceil(sz); % imresize equivalent.


if begstr(method,'cv')
    % will check params in detail.
    im2 = cn_imresize_opencv(im,sz,method);
else
    error('unknown method');
end

