function y = cn_linspace(from, to, n)
% y = cn_linspace(from, to, n)
%
% Just like Matlab's linspace, but, supports array of 
% from, to efficiently.
%
% INPUT
% from, to:     Arrays of the same length, matrices/nd-arrays are converted to vectors.
% [n]:          (default=100) a scalar.
% 
% OUTPUT
% y:            (numel(from) n) where y(i,:) = linspace(from(i),to(i),n)
% 
% R.G.Cinbis, Feb 2010

% $Id: cn_linspace.m,b 1.3 2013/03/26 09:08:51 cinbis Exp $

if nargin == 2
    n = 100;
end

% ensures that the sizes are the same
from = reshape(from,[],1);
to = reshape(to,size(from));
n = double(n);

y = [bsxfun(@plus,from,bsxfun(@times,(0:n-2)./(floor(n)-1),(to-from))) to];

