function info = cn_gaussdiag_precalc(p)
% Pre-calculate gaussians/GMM to get log-pb for many provided gaussians.
% Each gaussian has a diagonal covariance.
%
% INPUT
% p
%   .mu         (D K)   Means.
%   .vars       (D K)   Variances.
%   .[priors]   (1 K)   Prior probabilities of each gaussian. (default=1s)
%                       Not necessarily a distribution. Will not be normalized.
%
% OUTPUT
% info      A pre-calculated form for the gaussian.  
%
% SEE cn_gaussdiag_precalc cn_gaussdiag_pb
%
% todo: row-wise support.
%
% R.G.Cinbis July 2011

mu = p.mu;
vars = p.vars;

D = size(mu,1);
K = size(mu,2);
assert(size(vars,1)==D);
assert(size(vars,2)==K);

if isfield(p,'priors')
    priors = reshape(p.priors,1,K);
    assert(min(priors)>=0);
    assert(max(priors)<=1);
    logprior = log(priors+realmin);
else
    mywarn('Assumming priors(:)=1');
    logprior = zeros(1,K);
end

% find log-normalizer for each gaussian posterior
%normconst = log( ((2*pi).^(-D/2)) .* ( prod(vars,1).^(-1/2) ) );
normconst = (-D/2)*log(2*pi) + (-1/2)*sum(log(vars),1); % DONT +logprior
normconst = reshape(normconst,1,K);

% precision = 1 / var
precs = 1 ./ vars; % (D K)

% -- set info struct --

info.logprior = logprior;
info.K = K;
info.D = D;

if 0

    % old with cn_pdist2 
    info.mu = mu;
    info.precs = precs;
    info.normconst = normconst ;

else

    % for matlab-based alternative
    info.w_x = precs .* mu; % (D K)
    info.w_x2 = -0.5*precs; % (D K)
    nh_spmu2 = -0.5*sum(precs.*(mu.^2) , 1); % (1 K) to add += -.5 * sum_d( w_d * mu_d^2 )
    info.bias = bsxfun(@plus,normconst,nh_spmu2); % (1 K)

end


