classdef cn_onCleanup < handle
% Just like onCleanup, but allows changing "object.task" property.
% Set object.task=[] to cancel onCleanup action.
%
% This is a handle object, which calls "object.task" on the clean-up of the last reference. 

    properties(SetAccess='public', GetAccess='public', Transient)
        task;
    end
    
    methods

        function h = cn_onCleanup(functionHandle)
            % Constructor cn_onCleanup(task) where task is either [] or
            % a function handle.
            h.task = functionHandle;
        end
        
        function delete(h)
            try
                if ~isempty(h.task)
                    h.task();
                end
            catch e
                mywarn('---- UNALLOWED EXCEPTION CATCHED AT cn_onCleanup.delete() ----');
                mywarn(e.getReport());
                mywarn('------------------------------------------------------------------');
            end
        end

    end

end

