function cn_setrandseed(seed)
% cn_setrandseed(seed)
%
% Resets default RandStream object by the given seed. seed is normally a scalar.
%
% To use clock-based seed, use seed = 100*sum(clock).
% 
% If input is a string, it will be hashed and converted into a scalar (less than 2^32) to be used as a seed.
% For this, the first 7 hex characters (from-left) of md5 hash will be used.
%
% EXAMPLE
% % Get filename-specific randomization:
% cn_setrandseed(['file1' num2str(100)]); rand
% cn_setrandseed(['file2' num2str(100)]); rand
%
% SEE cn_loadrandstream cn_saverandstream CalcMD5
%
% R.G.Cinbis Jan 2010, Feb 2012

if ischar(seed)
    x = CalcMD5(seed,'md5'); 

    % Commented out on 2012-02-19. CalcMD5 seems fine.
    %
    % 2012-01-04: 
    % - moving to CalcMD5, supposed to be faster and java-free.
    % - sanity check for backward compatibility (OK so far). remove this check in the future.
    % if cn_isjvm 
    %    assert(isequal(x,kleder_hash(seed,'md5')),'inconsistency between kleder_hash and CalcMD5');
    %    mywarn('randseed:ok');
    % end

    x=x(1:7); % to meet 2^32 limit.
    seed=hex2dec(x);
end

s = RandStream.create('mt19937ar','seed',seed);

%RandStream.setDefaultStream(s);
RandStream.setGlobalStream( RandStream.create('mt19937ar','seed',seed) );

