function d = multicoredirs(name,newdir)
% Sets my default directory for the multicore parameter files.  Each directory
% is associated with a keyword "name". Directory for the same "name" can be overriden
% later.
%
% SUGGESTED
% % In startup.m, do:
% gokberk_multicoredirs('global','/home/lear/cinbis/multicore/');
% gokberk_multicoredirs('local','/dev/shm/cinbis/multicore/');
% % Then, for usage based on /dev/shm to reduce I/O overhead, use
% startmulticoremaster(fh,prms,struct2('multicoreDir','local'));
%
% R.G.Cinbis July 2011

persistent names dirs
mlock

if nargin == 2 
    newdir = cn_fixpath(newdir);
    if newdir(end)~='/'
        newdir(end+1)='/'; 
    end

    fprintf('Adding multicore directory %s: %s \n',name,newdir);
    i = find( strcmp(name,names) );
    if isempty(i)
        i = length(names) + 1;
    end
    names{i} = name;
    dirs{i}  = newdir;

    if ~exist(newdir,'dir') % create directory automatically
        mywarn(sprintf('Creating multicore directory %s:%s\n',name,newdir));
        mkdir(newdir);
    end
elseif nargin == 1 
    i = find(strcmp(names,name),1);
    if isempty(i)
        mywarn(sprintf('Cant find mc name %s, using the default one.',name));
        d = dirs{1};
    else
        d = dirs{i};
    end
else
    % return all directories in order. the very first one should be considered as the default.
    d = dirs;
end



