function format = cn_terminalcolor(format,color)
% Given a sprintf/fprintf format, it wraps it with color requested
% from the list. This works in Linux bash well.
%
% INPUT
% format    The first parameter for sprintf() or 1st/2nd for fprintf()
% color     Color requested.
%
% OUTPUT
% format       format wrapped with color.
%
% EXAMPLE
% fprintf(cn_colortext('helloworld-%d%s\n','boldyellow'),1,'a')
%
% REFERENCE
% http://stackoverflow.com/questions/1961209/making-some-text-in-printf-appear-in-green-and-red
%
% R.G.Cinbis May 2011

RESET = '\033[0m';

switch(upper(color))
    case 'BLACK'
       c='\033[30m';      % Black 
    case 'RED'
       c='\033[31m';      % Red 
    case 'GREEN'
       c='\033[32m';      % Green 
    case 'YELLOW'
       c='\033[33m';      % Yellow 
    case 'BLUE'
       c='\033[34m';      % Blue 
    case 'MAGENTA'
       c='\033[35m';      % Magenta 
    case 'CYAN'
       c='\033[36m';      % Cyan 
    case 'WHITE'
       c='\033[37m';      % White 
    case 'BOLDBLACK'
       c='\033[1m\033[30m';      % Bold Black 
    case 'BOLDRED'
       c='\033[1m\033[31m';      % Bold Red 
    case 'BOLDGREEN'
       c='\033[1m\033[32m';      % Bold Green 
    case 'BOLDYELLOW'
       c='\033[1m\033[33m';      % Bold Yellow 
    case 'BOLDBLUE'
       c='\033[1m\033[34m';      % Bold Blue 
    case 'BOLDMAGENTA'
       c='\033[1m\033[35m';      % Bold Magenta 
    case 'BOLDCYAN'
       c='\033[1m\033[36m';      % Bold Cyan 
    case 'BOLDWHITE'
       c='\033[1m\033[37m';      % Bold White 
    otherwise
        error('unknown color');
end

format = [c format RESET];

